CCCL (CUDA C++ Core Libraries) provides: - CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk) - Thrust: high-level parallel algorithms (transform_reduce, sort, scan) - libcudacxx: CUDA C++ standard library (atomics, barriers, memory) - cudax: experimental features (memory resources, allocators) - Tuning policies: per-SM hardware-specific algorithm parameters Competition optimization vectors mapped to CCCL: - Output TPS (83% weight): warp_reduce, block_reduce, device_topk - Input TPS (14% weight): device_scan, block_load, prefetch - Cache TPS (3% weight): prefix caching strategy patterns - Memory (0.9 util): pooled/cached/buddy allocators Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only) License: Apache-2.0
74 lines
1.6 KiB
Plaintext
74 lines
1.6 KiB
Plaintext
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of CUDASTF in CUDA C++ Core Libraries,
|
|
// under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
/**
|
|
* @file
|
|
*
|
|
* @brief An AXPY kernel described using a cuda_kernel construct
|
|
*
|
|
*/
|
|
|
|
#include <cuda/experimental/stf.cuh>
|
|
|
|
using namespace cuda::experimental::stf;
|
|
|
|
__global__ void axpy(double a, slice<const double> x, slice<double> y)
|
|
{
|
|
int tid = blockIdx.x * blockDim.x + threadIdx.x;
|
|
int nthreads = gridDim.x * blockDim.x;
|
|
|
|
for (int i = tid; i < x.size(); i += nthreads)
|
|
{
|
|
y(i) += a * x(i);
|
|
}
|
|
}
|
|
|
|
double X0(int i)
|
|
{
|
|
return sin((double) i);
|
|
}
|
|
|
|
double Y0(int i)
|
|
{
|
|
return cos((double) i);
|
|
}
|
|
|
|
int main()
|
|
{
|
|
context ctx = graph_ctx();
|
|
const size_t N = 16;
|
|
double X[N], Y[N];
|
|
|
|
for (size_t i = 0; i < N; i++)
|
|
{
|
|
X[i] = X0(i);
|
|
Y[i] = Y0(i);
|
|
}
|
|
|
|
double alpha = 3.14;
|
|
|
|
auto lX = ctx.logical_data(X);
|
|
auto lY = ctx.logical_data(Y);
|
|
|
|
/* Compute Y = Y + alpha X */
|
|
ctx.cuda_kernel(lX.read(), lY.rw())->*[&](auto dX, auto dY) {
|
|
// axpy<<<16, 128, 0, ...>>>(alpha, dX, dY)
|
|
return cuda_kernel_desc{axpy, 16, 128, 0, alpha, dX, dY};
|
|
};
|
|
|
|
ctx.finalize();
|
|
|
|
for (size_t i = 0; i < N; i++)
|
|
{
|
|
assert(fabs(Y[i] - (Y0(i) + alpha * X0(i))) < 0.0001);
|
|
assert(fabs(X[i] - X0(i)) < 0.0001);
|
|
}
|
|
}
|